###################################################################################
# nekter funkce predpokladaji urcitou konretni syntaxy SELECT:
#   SELECT
#       select_expression,...
#       [FROM table_references
#           [WHERE where_definition]
#           [GROUP BY {unsigned_integer | col_name | formula}]
#           [HAVING where_definition]
#           [ORDER BY {unsigned_integer | col_name | formula} [ASC | DESC] ,...]
#           [LIMIT [offset,] rows]
#   
###################################################################################

package Maccess_query_string;

use strict;
use lib ("/home/web-mycompany/www/maccess/bin","/home/web-mycompany/www/lib");

my $pkg="Maccess_query_string";

use Maccess_db;

#-- definice zakladnich promenych pro komunikaci s SQL servrem
my $dbh_sys=Maccess_db->dbh_sys;

#-- definice ruznych zakladnich promenych
my $white_space=" \t\x0A\x0D";
my $quotes=q{"'`};
my $left_brackets="([{";
my $quotes_brackets_left="$quotes$left_brackets";
my %quotes_brackets_pair=(
   "(" => ")",
   "[" => "]",
   "{" => "}",
   '"' => '"',
   "'" => "'",
   "`" => "`"
);
my $quotes_brackets=join("",keys(%quotes_brackets_pair)) . join("",values(%quotes_brackets_pair));
my $operations_operators="+-*/|&<>~!=%";
my $no_chars="$white_space$quotes_brackets$operations_operators,?:;\\@#^";   # tecka sem nepatri


#-------- Funkce, ktera z query_hash dela query_string
sub query_hash_to_query_string {
   my ($pkg_p, $query_hash)=@_;
   my $query_string;
   if(length($$query_hash{COLS})) {
      $query_string="SELECT $$query_hash{COLS}";
      if(length($$query_hash{FROM})) {
         $query_string.=" FROM $$query_hash{FROM}";
         if(length($$query_hash{WHERE})) {
            $query_string.=" WHERE $$query_hash{WHERE}";
         }
         if(length($$query_hash{"GROUP BY"})) {
            $query_string.=" GROUP BY $$query_hash{'GROUP BY'}";
         }
         if(length($$query_hash{HAVING})) {
            $query_string.=" HAVING $$query_hash{HAVING}";
         }
         if(length($$query_hash{"ORDER BY"})) {
            $query_string.=" ORDER BY $$query_hash{'ORDER BY'}";
         }
         if(length($$query_hash{LIMIT})) {
            $query_string.=" LIMIT $$query_hash{LIMIT}";
         }
      }
   }
   return $query_string;
}

#-------- Funkce vraci hash, ktery obsahuje zakladni casti query dotazu z $query_string (COLS,FROM,...)
sub query_string_to_query_hash {
   my ($pkg_p, $str)=@_;
   my %query_hash;
   if($str=~/^([\Q$white_space\E]*SELECT[\Q$white_space\E]+)/si) {
      my $end;
      my $pos=length($1);
      ($pos,$query_hash{COLS},$end)=$pkg->query_hash_string($str,$pos,"FROM",1);
      if(defined($end)) {
         ($pos,$query_hash{FROM},$end)=$pkg->query_hash_string($str,$pos,"WHERE|GROUP BY|HAVING|ORDER BY|LIMIT",1);
         if($end=~/^WHERE$/i) {
            ($pos,$query_hash{WHERE},$end)=$pkg->query_hash_string($str,$pos,"GROUP BY|HAVING|ORDER BY|LIMIT",1);
         }
         if($end=~/^GROUP BY$/i) {
            ($pos,$query_hash{"GROUP BY"},$end)=$pkg->query_hash_string($str,$pos,"HAVING|ORDER BY|LIMIT",1);
         }
         if($end=~/^HAVING$/i) {
            ($pos,$query_hash{HAVING},$end)=$pkg->query_hash_string($str,$pos,"ORDER BY|LIMIT",1);
         }
         if($end=~/^ORDER BY$/i) {
            ($pos,$query_hash{"ORDER BY"},$end)=$pkg->query_hash_string($str,$pos,"LIMIT",1);
         }
         if($end=~/^LIMIT$/i) {
            ($pos,$query_hash{LIMIT},$end)=$pkg->query_hash_string($str,$pos);
         }
      }
   }
   return (\%query_hash);
}

#-------- Pomocna funkce slouzici pro zjisteni urcite casti z $query_string
# pokud nebyl $end nalezen vraci cely retezec, ale $end_real je undef
sub query_hash_string {
   my ($pkg_p, $str,$pos,$end, $add_white_space,$encapsulate_only_funexpr)=@_;
   my $query_hash_string;
   my $encapsulate;
   if($encapsulate_only_funexpr) {
      $encapsulate=$quotes."[{";
   }
   else {
      $encapsulate=$quotes_brackets_left;
   }
   my $reg;
   if($add_white_space) {
      $reg='^([' . "\Q$white_space\E" . ']+)(' . $end . ')([' . "\Q$white_space\E" . ']+)';
   }
   else {
      $reg= '^()(' . $end . ')()';
   }
   my $str_length=length($str);
   my $pos_begin=$pos;
   my $end_real;
   while(defined(my $char=substr($str,$pos,1))) {
      ++$pos;
      if($char=~/([\Q$encapsulate\E])/s) {
         $pos=$pkg->quote_bracket($str,$pos,$quotes_brackets_pair{$1});  # vracena pozice ukazuje na znak za ukoncenim zavorky respektive uvozovky
      }
      elsif(defined($end) and substr($str,$pos-1)=~/$reg/si) {
         $query_hash_string=substr($str,$pos_begin,$pos-$pos_begin-1);
         $end_real=$2;
         $pos+=length($1.$2.$3)-1;
         last;
      }
      if($pos==$str_length) { # pozice ukazuje na "znak" za poslednim znakem v retezci
         $query_hash_string=substr($str,$pos_begin);
         last;
      }
      if($encapsulate_only_funexpr) {
         # zavorku "(" nezapouzdruj jen tehdy pokud ji predchazi prazdne znaky nebo "(" (a zaroven predtim nedoslo k zapouzdreni) - jinak ji zapouzdri
         if($char=~/([\Q$white_space(\E])/s and $char!~/([\Q$encapsulate\E])/s) {
            $encapsulate=$quotes."[{";
         }
         else {
            $encapsulate=$quotes_brackets_left;
         }
      }
   }
   return ($pos,$query_hash_string,$end_real);
}

#-------- Pomocna funkce slouzici k prochazeni $query_string a zajistujici, ze castech v zavorkach a uvozovkach nejsou hledana zakladni slova SELECT syntaxe
# jsem uvnitr a hledam uzaver - vratim pozici, ktera je za uzaverem
sub quote_bracket {
   my ($pkg_p, $str,$pos,$quote_bracket)=@_;
   while(defined(my $char=substr($str,$pos,1))) {
      ++$pos;
      if($char eq $quote_bracket) {
         last;
      }
      elsif($quote_bracket!~/[\Q$quotes\E]/s) {
         if($char=~/([\Q$left_brackets\E])/s) {
            $pos=$pkg->quote_bracket($str,$pos,$quotes_brackets_pair{$1});
         }
         elsif($char=~/([\Q$quotes\E])/s) {
            $pos=$pkg->quote_bracket($str,$pos,$1);
         }
      }
   }
   return $pos;
}

#-------- Ziskani seznamu tabulek z $query_string
# funkce predpoklada urcitou konretni syntaxy table_references:
#   table_reference INNER JOIN table_reference ON conditional_expr ...
#   table_reference LEFT OUTER JOIN table_reference ON conditional_expr ...
sub query_string_tables {
   my ($pkg_p, $str)=@_;
   my $query_hash=$pkg->query_string_to_query_hash($str);
   $str=$$query_hash{FROM};
   my $str_length=length($str);
   my @tables;
   my $pos=0;
   while($pos<$str_length) {
      my ($table_pom,$end);
      my ($table,$table_as);
      if($pos==0) {
         ($pos,$table_pom,$end)=$pkg->query_hash_string($str,$pos,"INNER JOIN|LEFT OUTER JOIN",1);
      }
      else {
         ($pos,$table_pom,$end)=$pkg->query_hash_string($str,$pos,"ON",1);
      }
      if($table_pom=~/([^\Q$no_chars\E]+)[\Q$white_space\E]+AS[\Q$white_space\E]+([^\Q$no_chars\E]+)$/si) {
         $table=$1;
         $table_as=$2;
      }
      elsif($table_pom=~/([^\Q$no_chars\E]+)$/si) {
         $table=$1;
      }
      if(defined($table)) {
         my ($database_clearname,$table_clearname);
         if($table=~/^(.+)\.(.+)$/) {
            $database_clearname=$1;
            $table_clearname=$2;
         }
         else {
            die "Bad table name: " . $table;
         }
         push(@tables,[$table,$table_as,$table_clearname,$database_clearname]);
      }
   }
   return (@tables);
}

#-------- Funkce na $query_string vraci pole tabulek v jednoduchem tvaru "databaze.tabulka"
# u update dotazu je nepristupne pojmenovavat tabulky: "table as tablepojm"
# bohuzel tim padem nedovoluji update dotaz pri spojeni dvou tabulek se stejnym nazvem, ale z ruznych DB
# proste beru v uvahu jen tabulky co nemaji pojmenovani a pak jen ty polozky u nichz je nalezena prislusna tabulka
sub query_string_tables_simple {
   my ($pkg_p, $str)=@_;
   my @tables_fullname;
   my @tables_pom=$pkg->query_string_tables($str);
   foreach my $table_a (@tables_pom) {
      if(!defined($$table_a[1])) {
         my $table_fullname="$$table_a[3].$$table_a[2]";
         if(!grep(/^\Q$table_fullname\E$/,@tables_fullname)) {
            push(@tables_fullname,$table_fullname);
         }
      }
   }
   return (@tables_fullname);
}

#-------- Ziskani seznamu sloupcu z $query_string
sub query_string_cols {
   my ($pkg_p, $str)=@_;
   my @tables=$pkg->query_string_tables($str);
   my $query_hash=$pkg->query_string_to_query_hash($str);
   $str=$$query_hash{COLS};
   my $str_length=length($str);
   my @cols;
   my $pos=0;
   my $table;
   while($pos<$str_length) {
      my ($col_pom,$end);
      my ($col,$col_as);
      ($pos,$col_pom,$end)=$pkg->query_hash_string($str,$pos,'[' . "\Q$white_space\E" . ']*,[' . "\Q$white_space\E" . ']*');
      my ($pos_as_p,$col_as_p,$end_as_p)=$pkg->query_hash_string($col_pom,0,"AS",1);
      if(defined($end_as_p)) {
         $col=$col_as_p;
         $col_as=substr($col_pom,$pos_as_p);
      }
      else {
         $col=$col_pom;
      }
      if(defined($col)) {
         my ($table_clearname,$col_clearname);
         if($col eq "*") {
            $col_clearname="*";
         }
         elsif($col=~/^([^\Q$no_chars.\E]+)\.\*$/) {
            $col_clearname="*";
            $table_clearname=$1;
         }
         elsif($col=~/^([^\Q$no_chars.\E]+)\.([^\Q$no_chars.\E]+)$/) {
            $col_clearname=$2;
            $table_clearname=$1;
         }
         elsif($col=~/^[^\Q$no_chars.\E]+$/ and $col!~/^\d+$/) { # predpokladam, ze ciste jmeno sloupce se nesmi skladat jen z cislic
            $col_clearname=$col;
         }
         push(@cols,[$col,$col_as,$col_clearname,$table_clearname,undef]);
      }
   }
   return (@cols);
}

#-------- Funkce na $query_string vraci pole sloupcu v jednoduchem tvaru "databaze.tabulka.sloupec"
sub query_string_cols_simple {
   my ($pkg_p, $str)=@_;
   my @cols_pom=$pkg->query_string_cols($str);
   my @cols_fullname=$pkg->cols_simple(\@cols_pom,$str);
   return @cols_fullname;
}

#-------- Funkce vracejici hash, kde klici i hodnotami jsou hodnoty sloupcu dle kterych se spojuje a take vracim pole ktere obsahuje kompletni seznam
## vracim hash - aby dotaz mohl byt update, musi byt na vystupu tisknut alespon sloupec, ktery se ve spojovacim vyrazu vyskytuje vlevo od rovnitka (ten vpravo tisten byt nemusi)
sub query_string_cols_join {
   my ($pkg_p, $str)=@_;
   my $query_hash=$pkg->query_string_to_query_hash($str);
   my @cols;
   my @cols_join_pom=split(/([^\Q$no_chars\E]+)=([^\Q$no_chars\E]+)/,$$query_hash{FROM});
   for(my $i=1;$i<=$#cols_join_pom;$i+=3) {
      for(my $j=0;$j<=1;++$j) {
         my $col=$cols_join_pom[$i+$j];
         my ($col_clearname,$table_clearname);
         if($col=~/^([^\Q$no_chars.\E]+)\.([^\Q$no_chars.\E]+)$/) {
            $col_clearname=$2;
            $table_clearname=$1;
         }
         elsif($col=~/^[^\Q$no_chars.\E]+$/) {
            $col_clearname=$col;
         }
         push(@cols,[undef,undef,$col_clearname,$table_clearname,undef]);
      }
   }
   my @cols_join_fullname=$pkg->cols_simple(\@cols,$str);
   my %cols_join_fullname;
   for(my $i=0;$i<$#cols_join_fullname;$i+=2) {
      $cols_join_fullname{$cols_join_fullname[$i+1]}=$cols_join_fullname[$i];
   }
   return \%cols_join_fullname;
}

#-------- Funkce jen pro zadany seznam sloupcu v $cols_pom vraci pole sloupcu v jednoduchem tvaru "databaze.tabulka.sloupec"
sub cols_simple {
   my ($pkg_p, $cols_pom,$str)=@_;
   my @cols_all_fullname=$pkg->cols_all($pkg->query_string_tables_simple($str));
   my @cols_fullname;
   foreach my $col_a (@$cols_pom) {
      my ($col,$col_as,$col_clearname,$table_clearname,$database_clearname)=@$col_a;
      if(defined($col_clearname)) {
         my $col_clearname_reg;
         if($col_clearname eq "*") {
            $col_clearname_reg=".+";
         }
         else {
            $col_clearname_reg="\Q$col_clearname\E";
         }
         my (@col_pom)=grep(/\.$col_clearname_reg$/i,@cols_all_fullname);  # zde ignoruji mala velka pismena
         my $table_clearname_reg;
         if(defined($table_clearname)) {
            $table_clearname_reg="\Q$table_clearname\E";
         }
         else {
            $table_clearname_reg=".+";
         }
         my (@col_pom)=grep(/\.$table_clearname_reg\./,@col_pom);
         if(defined(@col_pom)) {
            foreach my $col_pom (@col_pom) {
               ($database_clearname,$table_clearname,$col_clearname)=split(/\./,$col_pom);
               push(@cols_fullname,$database_clearname.".".$table_clearname.".".$col_clearname);
            }
         }
         else {
            ### ZMENA 2004-02-02 - vypoznamkovani radku nize; drive jsi zde koncil zesypanim; melo to zrejme ladici ucinek; ukazalo se, ze sem to normalne docestuje pri pojmenovavani tabulek v dotaze a neni duvod, aby se to zde asi hroutilo
            #die "Error \$col_clearname - asi to chce doladit fci query_string_cols pri ziskavani \$col_clearname.";
         }
      }
      else {
         push(@cols_fullname,undef);
      }
   }
   return (@cols_fullname);
}

#-------- Funkce vraci seznam vsech sloupcu pro zadane tabulky v poly @tables_fullname a to jednoduchem tvaru "databaze.tabulka.sloupec"
sub cols_all {
   my ($pkg_p, @tables_fullname)=@_;
   my @cols_all_fullname;
   foreach my $table_fullname (@tables_fullname) {
      my $sth=$dbh_sys->prepare(q{
         show
            columns
         from
            } . $table_fullname . q{
      });  ## tento zpusob se mi moc nelibi
      my $rv=$sth->execute;
      while(my $ref=$sth->fetchrow_arrayref) {
         push(@cols_all_fullname,"$table_fullname.$$ref[0]");
      }
      my $rc=$sth->finish;
   }
   return @cols_all_fullname;
}

#-------------- Funkce zjistujici primarni klice vsech tabulek v $query_string a vraci je ve tvaru "databaze.tabulka.sloupec"
sub query_string_primary_keys {
   my ($pkg_p, $str)=@_;
   my @primary_keys;
   my @tables_fullname=$pkg->query_string_tables_simple($str);
   foreach my $table_fullname (@tables_fullname) {
      push(@primary_keys,$pkg->primary_keys($table_fullname));
   }
   return @primary_keys;
}

#-------------- Funkce zjistujici primarni klice tabulky
sub primary_keys {
   my ($pkg_p, $table_fullname)=@_; # table predpokladam ve tvaru database.table
   my @primary_keys;
   my $sth=$dbh_sys->prepare(q{
      show
         columns
      from
         } . $table_fullname . q{
   });  ## tento zpusob se mi moc nelibi
   my $rv=$sth->execute;
   while(my $ref=$sth->fetchrow_arrayref) {
      if($$ref[3] eq "PRI") {
         push(@primary_keys, "$table_fullname.$$ref[0]");
      }
   }
   my $rc=$sth->finish;
   return @primary_keys;
}

#-------------- Funkce, ktera zjistuje zda jde o souhny dotaz - tedy zda mohou byt radky odgrupovavany
sub may_be_ungroup_query {
   my ($pkg_p, $str)=@_;
   my $may_be=0;
   my $query_hash=$pkg->query_string_to_query_hash($str);
   if(length($$query_hash{"GROUP BY"})) { ## neni uplne ok - souhrny dotaz nemusi obsahovat group by
      $may_be=1;
   }
   else {
      $may_be=0;
   }
   return $may_be;
}

#-- Ziskani $query_string bez polozky limit a navrat limitujicich hodnot v puvodnim $query_string
sub query_string_no_limit {
   my ($pkg_p, $query_string)=@_;
   my $query_hash=$pkg->query_string_to_query_hash($query_string);
   my @limit;
   if(length($$query_hash{LIMIT})) {
      my $limit=$$query_hash{LIMIT};
      $$query_hash{LIMIT}=undef;
      $query_string=$pkg->query_hash_to_query_string($query_hash);
      $limit=~s/[\Q$white_space\E]//gs;
      @limit=split(/,/,$limit);
      if(!defined($limit[1])) {
         $limit[1]=$limit[0];
         $limit[0]=0;
      }
   }
   return ($query_string,@limit);
}

#--------------- Funkce vracej.retezec pro "select" dotaz na jednotlive zaznami vramci group datazu --#
## obdobnou vec delas ve fci Maccess->query_hash_to_ungroup_hash - casem by meli fce nejak scasti splynout
sub query_string_subgrp {
   my ($pkg_p, $query_string,$names_hash)=@_;
   my (%names_hash_subgrp);
   my @cols_group=$pkg->query_string_cols_group($query_string);
   my $group_where_string;
   foreach my $col_group (@cols_group) {
      my $col_group_as=$col_group;
      my @cols_pom=$pkg->query_string_cols($query_string);
      foreach my $col_a (@cols_pom) {
         if($col_group eq $$col_a[1]) {
            $col_group=$$col_a[0];
            $names_hash_subgrp{$col_group_as}=$$names_hash{$col_group_as};
            last;
         }
      }
      $group_where_string.="$col_group=\"$$names_hash{$col_group_as}\" and ";
   }
   $group_where_string=~s/ and $//si;
   my $query_hash=$pkg->query_string_to_query_hash($query_string);
   $$query_hash{COLS}="*";
   if(length($$query_hash{WHERE})) {
      $$query_hash{WHERE}="($$query_hash{WHERE}) and ($group_where_string)";
   }
   else {
      $$query_hash{WHERE}="($group_where_string)";
   }
   $$query_hash{"GROUP BY"}=undef;
   $$query_hash{HAVING}=undef;
   $$query_hash{"ORDER BY"}=undef;
   $$query_hash{LIMIT}=undef;
   $query_string=$pkg->query_hash_to_query_string($query_hash);
   return ($query_string,\%names_hash_subgrp);
}

#--------------- Funkce vraci jmena sloupcu (vlastne jejich pojmenovani "as") dle kterych je groupovano -#
sub query_string_cols_group {
   my ($pkg_p, $query_string)=@_;
   my @cols_group;
   my $query_hash=$pkg->query_string_to_query_hash($query_string);
   my $str=$$query_hash{"GROUP BY"};
   my $str_length=length($str);
   my $pos=0;
   while($pos<$str_length) {
      my ($col_group,$end);
      ($pos,$col_group,$end)=$pkg->query_hash_string($str,$pos,'[' . "\Q$white_space\E" . ']*,[' . "\Q$white_space\E" . ']*');
      if(defined($col_group)) {
         push(@cols_group,$col_group);
      }
   }
   return @cols_group;
}

#--------------- Funkce pro upravu podminky where a having ---------------------#
sub modify_Field_where {
   my ($pkg_p, $str,$Field_name)=@_;
   my $Field_where;
   foreach my $i (0,1) {
      my @operators;
      if(!$i) {
         $str=" $str"; # pridani pomocne mezery
         @operators=("<>","!=","<=",">=","=","<",">");
      }
      else {
         $str=$Field_where;
         $Field_where="";
         @operators=("NOT LIKE ","LIKE ","NOT REGEXP ","REGEXP ","NOT IN ","IN ","NOT IN(","IN(","IS NOT NULL","IS NULL");
      }
      my $str_length=length($str);
      my $reg_operators;
      foreach my $operator (@operators) {
         $reg_operators.= '[' . "\Q$white_space(\E" . ']' . "\Q$operator\E";
         $reg_operators.="|";
      }
      chop($reg_operators);
      my $pos=0;
      while($pos<$str_length) {
         my ($Field_where_pom,$end);
         # temer vse je zapouzdrovano: if(a like "*",1,0); >(1=1); =(a like "*")
         ($pos,$Field_where_pom,$end)=$pkg->query_hash_string($str,$pos,$reg_operators,0,1);
         if(defined($end)) {
            $end= substr($end,0,1) . $Field_name . (" " x $i) . substr($end,1);
            $Field_where.="$Field_where_pom$end";
         }
         else {
            $Field_where.=$Field_where_pom;
         }
      }
   }
   $Field_where=~s/^ //s; # odstraneni pomocne mezery
   return $Field_where;
}

#--------------- Nektere upravy query stringu (v like *,? -> %,_; uprava datumu)--#
## zadna krasavice
sub convert_some_strings {
   my ($pkg_p, $query_string)=@_;
   #--- predelani like "*?" -> like "%_"
   $query_string=~s/([\Q$white_space\E]LIKE[\Q$white_space\E]+)([\Q$quotes\E])(.+?)\2/ $1 . $2 . $pkg->correct_like($3) . $2 /gsie;
   #--- predelani datumu do tvaru yyyy-mm-dd (a to napriklad z tvaru yyyy-m-d nebo d.m.yyyy)
   $query_string=~s/([\Q$quotes\E])(\d{1,2}\.\d{1,2}\.\d{4}|\d{4}\-\d{1,2}-\d{1,2})\1/ $1 . $pkg->correct_date($2) . $1 /gse;
   return $query_string;
}

#-------------- Funkce ktera opravuje like: like "*?" -> like "%_"
sub correct_like {
   my ($pkg_p, $str)=@_;
   $str=~s/([%_])/\\$1/gs;
   $str=~s/\*/\%/gs;
   $str=~s/\?/\_/gs;
   return $str;
}

#-------------- Funkce ktera opravuje datum do tvaru yyyy-mm-dd (a to napriklad z tvaru yyyy-m-d nebo d.m.yyyy)
sub correct_date {
   my ($pkg_p, $date)=@_;
   $date=~s/^(\d{4})\-(\d{1,2})-(\d{1,2})$/ $1 . "-" . sprintf("%2.2d",$2) . "-" . sprintf("%2.2d",$3) /e;
   $date=~s/^(\d{1,2})\.(\d{1,2})\.(\d{4})$/ $3 . "-" . sprintf("%2.2d",$2) . "-" . sprintf("%2.2d",$1) /e;
   return $date;
}

#-------------- Obsahuje query_string promene?
## zadna krasavice
sub is_variables {
   my ($pkg_p, $query_string)=@_;
   my $variables;
   if($query_string=~/\[([^\]:]+:[^\]]*)\]/s) {
      $variables=1;
   }
   else {
      $variables=0;
   }
   return $variables;
}

#--------------- Doplneni hodnot promenych do query_string ---------------------#
## zadna krasavice - obdobna cast viz Maccess->query_form a take viz vyse na is_variables
sub insert_values_to_query_string {
   my ($pkg_p, $query_string,$variables)=@_;
   my @variables_insert; # seznam promenych, ktere byly skutecne do $query_string vlozeny
   my @variables_insert_names;
   my @query_string_split=split(/\[([^\]:]+:[^\]]*)\]/si,$query_string);
   $query_string="";
   for(my $i=0;$i<=$#query_string_split;++$i) {
      if($i % 2) { # kazde druhe (liche)
         my @variable=split(/:/si,$query_string_split[$i],2);
         foreach my $variable_val (@$variables) {
            if($variable[0] eq $$variable_val[0]) {
               $query_string_split[$i]=$$variable_val[1];
               if(!grep(/^\Q$$variable_val[0]\E$/,@variables_insert_names)) {
                  push(@variables_insert_names,$$variable_val[0]);
                  push(@variables_insert,[$$variable_val[0],$$variable_val[1],$$variable_val[2]]);
               }
               last;
            }
         }
      }
      $query_string.=$query_string_split[$i];
   }
   return ($query_string,\@variables_insert);
}

return 1;
